/* -*- c-basic-offset: 4; -*-
 * Copyright (c) 2005 Hans Insulander
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: audio.cc,v 1.5 2005/11/10 14:25:39 hin Exp $
 */

#include <math.h>

#include "common.hh"

PortAudioStream *inputstream = NULL;

int in_channels = 2;
int out_channels = 2;

float
magnitude(float re, float im) {
    return sqrtf(re*re + im*im);
}

float outbuf_re[FFTLEN/2];
float outbuf_im[FFTLEN/2];

volatile float *fftdata_re = NULL;
volatile float *fftdata_im = NULL;

double
hamming(double n) {
    return 0.54 - 0.46*cos(2*M_PI*n);
}

int
recordCallback(void *inputBuffer, void *outputBuffer,
	       unsigned long framesPerBuffer,
	       PaTimestamp outTime, void *userData) {
    float *in = (float *)inputBuffer;
    //    float *out = (float *)outputBuffer;
    unsigned int i;
    if(in == NULL)
	return 0;

    if(fftdata_re != NULL) {
	return 0;
    }

    fflush(stdout);
    float re[FFTLEN];
    float im[FFTLEN];

    for(i=0; i<framesPerBuffer; i++) {
	static int t = 0;
	t++;
	float s = 0;
	for(int c=0; c<in_channels; c++)
	    s += *in++;
	s /= (float)in_channels;
	re[i] = s*hamming((double)i/(double)FFTLEN);
	im[i] = 0;
    }

    fft(1, (int)(log(FFTLEN)/log(2)), re, im);

    for(i=0; i<FFTLEN/2; i++) {
	for(int c=0; c<out_channels; c++) {
	    outbuf_re[i] = re[i];
	    outbuf_im[i] = im[i];
	}
    }

    fftdata_re = outbuf_re;
    fftdata_im = outbuf_im;
    return 0;
}

void
audio_init() {
    PaError result;
    result = Pa_Initialize();
    if(result != paNoError) {
	printf("PortAudio error: %s\n", Pa_GetErrorText(result));
    }
    result = Pa_OpenDefaultStream(&inputstream,
				  in_channels, /* input channels */
				  out_channels, /* output channels */
				  paFloat32,
				  SAMPLERATE,
				  FFTLEN,
				  0,
				  recordCallback,
				  NULL);
    if(result != paNoError) {
	printf("PortAudio error: %s\n", Pa_GetErrorText(result));
    }

    result = Pa_StartStream(inputstream);
    if(result != paNoError) {
	printf("PortAudio error: %s\n", Pa_GetErrorText(result));
    }

    printf("Audio OK\n");
}
