/* -*- c-basic-offset: 4; -*-
 *
 * $Id: fft.cc,v 1.3 2005/11/10 14:25:39 hin Exp $
 *
 * Taken from http://astronomy.swin.edu.au/~pbourke/analysis/dft/
 *
 * I asked if i could use this code in a mail to the author. Here is
 * the response:
 *
 *	From: Paul Bourke <pdb@swin.edu.au>
 *	Subject: Re: fft code use
 *	To: Hans Insulander <hin@hin.nu>
 *	Date: Sat, 1 May 2004 09:55:49 +1000
 *
 *	> So I would like to have your permission to use your code in my personal
 *	> investigation in audio processing and synthesis, but it would be
 *	> nice if
 *	> I could use that code in the (very unlikely) event that I want to make
 *	> a commercial product including fft.
 *
 *	Permission granted to use it in personal or commercial works as long as
 *	appropriate credits are given to myself.
 *
 *	-------------------------------------
 *	P a u l   B o u r k e
 *	Astrophysics and Supercomputing
 *	email   pdb@swin.edu.au
 *	www     astronomy.swin.edu.au/~pbourke
 *	phone   +61 -3 9214 8624
 *	fax     +61 -3 9214 8797
 *	aim     bpdourke
 *	letter  Mail number 31
 *	         PO Box 218 Hawthorn
 *	         Swinburne University of Technology
 *	         Victoria 3122, Australia
 *
 */

#include <sys/time.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

/*
 * This computes an in-place complex-to-complex FFT 
 * x and y are the real and imaginary arrays of 2^m points.
 * dir =  1 gives forward transform
 * dir = -1 gives reverse transform 
 */
void
fft(int dir, int m, float *x, float *y)
{
    long n,i,i1,j,k,i2,l,l1,l2;
    double c1,c2,tx,ty,t1,t2,u1,u2,z;

    /* Calculate the number of points */
    n = 1;
    for (i=0;i<m;i++) 
	n *= 2;

    /* Do the bit reversal */
    i2 = n >> 1;
    j = 0;
    for (i=0;i<n-1;i++) {
	if (i < j) {
	    tx = x[i];
	    ty = y[i];
	    x[i] = x[j];
	    y[i] = y[j];
	    x[j] = tx;
	    y[j] = ty;
	}
	k = i2;
	while (k <= j) {
	    j -= k;
	    k >>= 1;
	}
	j += k;
    }

    /* Compute the FFT */
    c1 = -1.0; 
    c2 = 0.0;
    l2 = 1;
    for (l=0;l<m;l++) {
	l1 = l2;
	l2 <<= 1;
	u1 = 1.0; 
	u2 = 0.0;
	for (j=0;j<l1;j++) {
	    for (i=j;i<n;i+=l2) {
		i1 = i + l1;
		t1 = u1 * x[i1] - u2 * y[i1];
		t2 = u1 * y[i1] + u2 * x[i1];
		x[i1] = x[i] - t1; 
		y[i1] = y[i] - t2;
		x[i] += t1;
		y[i] += t2;
	    }
	    z =  u1 * c1 - u2 * c2;
	    u2 = u1 * c2 + u2 * c1;
	    u1 = z;
	}
	c2 = sqrt((1.0 - c1) / 2.0);
	if (dir == 1) 
	    c2 = -c2;
	c1 = sqrt((1.0 + c1) / 2.0);
    }

    /* Scaling for forward transform */
    if (dir == 1) {
	for (i=0;i<n;i++) {
	    x[i] /= n;
	    y[i] /= n;
	}
    }
    return;
}
